@@ -10,7 +10,7 @@ use Search::Tools;    # XS required
 use Carp;
 use base qw( Search::Tools::TokenListUtils );
 
-our $VERSION = '0.28';
+our $VERSION = '0.29';
 
 1;
 
@@ -125,11 +125,16 @@ Prints internal XS attributes to stderr.
 
 =head2 get_heat
 
-Returns an array ref to the internal AV (array) of tokens with
+Returns an array ref to the internal AV (array) of positions with
 is_hot() set by the original Tokenizer. This method will return an
 empty list unless you have passed a heat_seeker to the tokenize() method.
 See Search::Tools::Tokenizer.
 
+=head2 get_sentence_starts
+
+Returns an array ref to the internal AV (array) of sentence start
+positions for each position in get_heat().
+
 =head2 matches
 
 Returns an array ref of all the Tokens with is_match() set. The
