@@ -8,7 +8,7 @@ use Search::Tools::TokenList;
 use Search::Tools::UTF8;
 use Carp;
 
-our $VERSION = '0.28';
+our $VERSION = '0.29';
 
 __PACKAGE__->mk_accessors(qw( re ));
 
@@ -47,6 +47,7 @@ sub tokenize_pp {
     my $heat_seeker_is_coderef
         = ( defined $heat_seeker and ref($heat_seeker) eq 'CODE' ) ? 1 : 0;
 
+    # TODO is_sentence_* logic
     for ( split( m/($re)/, $_[0] ) ) {
         next unless length($_);
         my $tok = bless(
@@ -135,11 +136,17 @@ you set this once in new(). The default value is:
 
 which will match words and contractions (e.g., "do", "not" and "don't").
 
-=head2 tokenize( I<string> )
+=head2 tokenize( I<string> [, I<heat_seeker>, I<match_num>] )
 
 Returns a TokenList object representin the Tokens in I<string>.
 I<string> is "split" according to the regex in re().
 
+I<heat_seeker> can be either a CODE reference or a regex object (qr//)
+to use for testing is_hot per token.
+
+I<match_num> is the parentheses number to consider the matching token
+in the re() value. The default is 0 (the entire matching pattern).
+
 =head2 tokenize_pp( I<string> )
 
 Returns a TokenListPP object.
