@@ -20,9 +20,9 @@ our @EXPORT = qw(
 
 );
 
-our $Debug = $ENV{PERL_DEBUG} || 0;
+our $Debug = ( $ENV{PERL_DEBUG} && $ENV{PERL_DEBUG} > 2 ) ? 1 : 0;
 
-our $VERSION = '0.28';
+our $VERSION = '0.29';
 
 sub to_utf8 {
     my $str = shift;
